// rcvUDPWin.cpp (V1.0): the server of UDP Load in MS windows.
//
// Author:  Huahui Wu
//          flashine@gmail.com
//
// History
//   Version 1.0 - 2005 Oct 11
//
// Description:
//   rcvUDPWin.cpp
//		Very simple, Works in conjunction with sndUDPWin.cpp as 
//		a Windows sender or sndUDPLnx as a Linux sender.
//		The program sets itself up as a server using the UDP
//		protocol. It waits for UDP data from a sender and reports
//		the receiving bitrate
//
// Compile and Link: 
//	 Use MS Visual C++ .NET
//	 Compile and link with wsock32.lib
//
// Run: 
//   Pass the port number that the server should bind() to
//   on the command line. Any port number not already in use
//   can be specified.
//	 Example: rcvUDPWin 2000
//
// License: 
//   This software is released into the public domain.
//   You are free to use it in any way you like.
//   This software is provided "as is" with no expressed
//   or implied warranty.  I accept no liability for any
//   damage or loss of business that this software may cause.
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "winsock.h"
#include <signal.h>

#define MAX_PKT_SIZE 10240
#define MAX_PKT_NUM 1000

struct Ping_Pkt{
	int seq;
	int nPkt;
	unsigned char padding[MAX_PKT_SIZE];
};

// Function prototype
void DatagramServer(short nPort);	// the server 
void CleanUp(int);						// handle for ctrl_c	
ULARGE_INTEGER GetCurTimeInMs(); //get current time in milli second


// Helper macro for displaying errors
#define PRINTERROR(s)	\
		fprintf(stderr,"\n%s: %d\n", s, WSAGetLastError())

int pktRcvd;
ULARGE_INTEGER initCounter; // the counter when PING starts
ULARGE_INTEGER prevCounter; // the counter of previous packet
ULARGE_INTEGER	curCounter;
ULARGE_INTEGER   frequency;
int minDelay, maxDelay; // the minum and the maximum delay

void main(int argc, char **argv)
{
	WORD wVersionRequested = MAKEWORD(1,1);
	WSADATA wsaData;
	int nRet;
	short nPort = 1234;
	pktRcvd = 0;
	initCounter.QuadPart = 0;
	prevCounter.QuadPart = 0;

	//
	// Check for port argument
	//
	if (argc > 2)
	{
		fprintf(stderr,"\nUsage: %s PortNumber\n", argv[0]);
		fprintf(stderr,"\n or %s\n", argv[0]);
		fprintf(stderr,"\t use default port number 1234");
		return;
	}
	if (argc == 2) nPort = atoi(argv[1]);
	
	//
	// Initialize WinSock and check version
	//
	nRet = WSAStartup(wVersionRequested, &wsaData);
	if (wsaData.wVersion != wVersionRequested)
	{	
		fprintf(stderr,"\n Wrong version\n");
		return;
	}


	signal(SIGINT, CleanUp);
	//
	// Do all the stuff a datagram server does
	//
	DatagramServer(nPort);
	
	//
	// Release WinSock
	//
	WSACleanup();
}

////////////////////////////////////////////////////////////

void DatagramServer(short nPort)
{
	//
	// Create a UDP/IP datagram socket
	//
	SOCKET theSocket;

	theSocket = socket(AF_INET,		// Address family
					   SOCK_DGRAM,  // Socket type
					   IPPROTO_UDP);// Protocol
	if (theSocket == INVALID_SOCKET)
	{
		PRINTERROR("socket()");
		return;
	}

	
	//
	// Fill in the address structure
	//
	SOCKADDR_IN saServer;

	saServer.sin_family = AF_INET;
	saServer.sin_addr.s_addr = INADDR_ANY; // Let WinSock assign address
	saServer.sin_port = htons(nPort);	   // Use port passed from user


	//
	// bind the name to the socket
	//
	int nRet;

	nRet = bind(theSocket,				// Socket descriptor
				(LPSOCKADDR)&saServer,  // Address to bind to
				sizeof(struct sockaddr)	// Size of address
				);
	if (nRet == SOCKET_ERROR)
	{
		PRINTERROR("bind()");
		closesocket(theSocket);
		return;
	}


	//
	// This isn't normally done or required, but in this 
	// example we're printing out where the receiver is waiting
	// so that you can connect the example sender.
	//
	int nLen;
	nLen = sizeof(SOCKADDR);
	char szBuf[MAX_PKT_SIZE];

	nRet = gethostname(szBuf, sizeof(szBuf));
	if (nRet == SOCKET_ERROR)
	{
		PRINTERROR("gethostname()");
		closesocket(theSocket);
		return;
	}

	//
	// Show the server name and port number
	//
	printf("\nServer named %s waiting on port %d\n",
			szBuf, nPort);
			

	//
	// Wait for data from the sender
	//
	SOCKADDR_IN saClient;
	double tInterval; // the difference between the time when two packets were received

	while(1){
		memset(szBuf, 0, sizeof(szBuf));
		nRet = recvfrom(theSocket,				// Bound socket
						szBuf,					// Receive buffer
						sizeof(szBuf),			// Size of buffer in bytes
						0,						// Flags
						(LPSOCKADDR)&saClient,	// Buffer to receive sender address 
						&nLen);					// Length of sender address buffer

		Ping_Pkt* ping_pkt=(Ping_Pkt*)szBuf; 
		if (ping_pkt->seq < 0) //the sender stopped
			CleanUp(1);

		//
		// Display the data that was received once per second
		//
		if (ping_pkt->seq % ping_pkt->nPkt == 0){
			curCounter = GetCurTimeInMs();

			if (initCounter.QuadPart == 0){ //if this is the first packet
				initCounter = curCounter;
			}

			//get the time between two display
			tInterval = curCounter.QuadPart - prevCounter.QuadPart;
			if (tInterval < 1)
				tInterval = 1;

			printf("From sender: size = %d bytes pkt recved=%d  bw=%8.3f KB\n",
				nRet, pktRcvd, 1.0*ping_pkt->nPkt*nRet/tInterval);

			prevCounter = curCounter;

		}
		pktRcvd ++;

	}
	closesocket(theSocket);
	return;
}

void CleanUp(int arg1){
	curCounter = GetCurTimeInMs();
	double tTotal = curCounter.QuadPart - initCounter.QuadPart;
	printf("Server is now off\n");
	printf("\t%d packets are received in %8.3f milli second\n", pktRcvd, tTotal);
	printf("\tAverage receiving bandwidth is %8.3f packets per second\n", 1000*pktRcvd/tTotal);
	exit(0);
}

ULARGE_INTEGER GetCurTimeInMs(){
	SYSTEMTIME  sysTime;
	FILETIME	fileTime;
	ULARGE_INTEGER timeInMs;
	::GetSystemTime(&sysTime);
	::SystemTimeToFileTime(&sysTime,&fileTime);
	timeInMs.LowPart = fileTime.dwLowDateTime;
	timeInMs.HighPart = fileTime.dwHighDateTime;
	timeInMs.QuadPart /= 10000; // since the filetime is 100-nanosecond based.
								// but we need milli second based
	return timeInMs;
}
